@@ -24,6 +24,10 @@ from spyder_kernels.comms.commbase import CommBase, CommError
 from spyder_kernels.py3compat import TimeoutError, PY2
 
 
+if PY2:
+    import thread
+
+
 def get_free_port():
     """Find a free port on the local machine."""
     sock = socket.socket()
@@ -34,7 +38,7 @@ def get_free_port():
     return port
 
 
-def frontend_request(blocking=True, timeout=None):
+def frontend_request(blocking, timeout=None):
     """
     Send a request to the frontend.
 
@@ -64,7 +68,7 @@ class FrontendComm(CommBase):
         self.register_call_handler('_send_comm_config',
                                    self._send_comm_config)
 
-        self.comm_lock = threading.Lock()
+        self.comm_lock = threading.RLock()
 
         # self.kernel.parent is IPKernelApp unless we are in tests
         if self.kernel.parent:
@@ -259,24 +263,31 @@ class FrontendComm(CommBase):
 
     def _remote_callback(self, call_name, call_args, call_kwargs):
         """Call the callback function for the remote call."""
-        saved_stdout_write = sys.stdout.write
-        saved_stderr_write = sys.stderr.write
-        sys.stdout.write = WriteWrapper(saved_stdout_write, call_name)
-        sys.stderr.write = WriteWrapper(saved_stderr_write, call_name)
-        try:
-            return super(FrontendComm, self)._remote_callback(
-                call_name, call_args, call_kwargs)
-        finally:
-            sys.stdout.write = saved_stdout_write
-            sys.stderr.write = saved_stderr_write
+        with self.comm_lock:
+            current_stdout = sys.stdout
+            current_stderr = sys.stderr
+            saved_stdout_write = current_stdout.write
+            saved_stderr_write = current_stderr.write
+            thread_id = thread.get_ident() if PY2 else threading.get_ident()
+            current_stdout.write = WriteWrapper(
+                saved_stdout_write, call_name, thread_id)
+            current_stderr.write = WriteWrapper(
+                saved_stderr_write, call_name, thread_id)
+            try:
+                return super(FrontendComm, self)._remote_callback(
+                    call_name, call_args, call_kwargs)
+            finally:
+                current_stdout.write = saved_stdout_write
+                current_stderr.write = saved_stderr_write
 
 
 class WriteWrapper(object):
     """Wrapper to warn user when text is printed."""
 
-    def __init__(self, write, name):
+    def __init__(self, write, name, thread_id):
         self._write = write
         self._name = name
+        self._thread_id = thread_id
         self._warning_shown = False
 
     def is_benign_message(self, message):
@@ -293,6 +304,10 @@ class WriteWrapper(object):
 
     def __call__(self, string):
         """Print warning once."""
+        thread_id = thread.get_ident() if PY2 else threading.get_ident()
+        if self._thread_id != thread_id:
+            return self._write(string)
+
         if not self.is_benign_message(string):
             if not self._warning_shown:
                 self._warning_shown = True
